@@ -1,15 +1,20 @@
 package diff
 
 import (
+	"strings"
+
 	"github.com/charmbracelet/bubbles/key"
 	"github.com/charmbracelet/bubbles/viewport"
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/idursun/jjui/internal/config"
 	"github.com/idursun/jjui/internal/ui/common"
-	"strings"
 )
 
+var _ common.Model = (*Model)(nil)
+
 type Model struct {
+	*common.ViewNode
+	*common.MouseAware
 	view   viewport.Model
 	keymap config.KeyMappings[key.Binding]
 }
@@ -33,32 +38,45 @@ func (m *Model) SetHeight(h int) {
 	m.view.Height = h
 }
 
-func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
+func (m *Model) Scroll(delta int) tea.Cmd {
+	if delta > 0 {
+		m.view.ScrollDown(delta)
+	} else if delta < 0 {
+		m.view.ScrollUp(-delta)
+	}
+	return nil
+}
+
+func (m *Model) Update(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
 	case tea.KeyMsg:
 		switch {
 		case key.Matches(msg, m.keymap.Cancel):
-			return m, common.Close
+			return common.Close
 		}
 	}
 	var cmd tea.Cmd
 	m.view, cmd = m.view.Update(msg)
-	return m, cmd
+	return cmd
 }
 
 func (m *Model) View() string {
+	m.view.Height = m.Height
+	m.view.Width = m.Width
 	return m.view.View()
 }
 
-func New(output string, width int, height int) *Model {
-	view := viewport.New(width, height)
+func New(output string) *Model {
+	view := viewport.New(0, 0)
 	content := strings.ReplaceAll(output, "\r", "")
 	if content == "" {
 		content = "(empty)"
 	}
 	view.SetContent(content)
 	return &Model{
-		view:   view,
-		keymap: config.Current.GetKeyMap(),
+		ViewNode:   common.NewViewNode(0, 0),
+		MouseAware: common.NewMouseAware(),
+		view:       view,
+		keymap:     config.Current.GetKeyMap(),
 	}
 }
