@@ -4,24 +4,15 @@ import (
 	"strings"
 	"time"
 
-	"github.com/idursun/jjui/internal/screen"
-
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/charmbracelet/lipgloss"
+	"github.com/charmbracelet/x/cellbuf"
+	"github.com/idursun/jjui/internal/config"
 	"github.com/idursun/jjui/internal/ui/common"
 	"github.com/idursun/jjui/internal/ui/context"
+	"github.com/idursun/jjui/internal/ui/intents"
 )
 
-const expiringMessageTimeout = 4 * time.Second
-
-type Model struct {
-	context      *context.MainContext
-	messages     []flashMessage
-	successStyle lipgloss.Style
-	errorStyle   lipgloss.Style
-	currentId    uint64
-}
-
 type expireMessageMsg struct {
 	id uint64
 }
@@ -33,12 +24,29 @@ type flashMessage struct {
 	id      uint64
 }
 
+type FlashMessageView struct {
+	// Content might contain ANSI colour codes
+	Content string
+	Rect    cellbuf.Rectangle
+}
+
+type Model struct {
+	*common.ViewNode
+	context      *context.MainContext
+	messages     []flashMessage
+	successStyle lipgloss.Style
+	errorStyle   lipgloss.Style
+	currentId    uint64
+}
+
 func (m *Model) Init() tea.Cmd {
 	return nil
 }
 
-func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
+func (m *Model) Update(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
+	case intents.Intent:
+		return m.handleIntent(msg)
 	case expireMessageMsg:
 		for i, message := range m.messages {
 			if message.id == msg.id {
@@ -46,58 +54,89 @@ func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
 				break
 			}
 		}
-		return m, nil
+		return nil
 	case common.CommandCompletedMsg:
 		id := m.add(msg.Output, msg.Err)
 		if msg.Err == nil {
-			return m, tea.Tick(expiringMessageTimeout, func(t time.Time) tea.Msg {
-				return expireMessageMsg{id: id}
-			})
+			expiringMessageTimeout := config.GetExpiringFlashMessageTimeout(config.Current)
+			if expiringMessageTimeout > time.Duration(0) {
+				return tea.Tick(expiringMessageTimeout, func(t time.Time) tea.Msg {
+					return expireMessageMsg{id: id}
+				})
+			}
 		}
-		return m, nil
+		return nil
 	case common.UpdateRevisionsFailedMsg:
 		m.add(msg.Output, msg.Err)
 	}
-	return m, nil
+	return nil
 }
 
-func (m *Model) View() string {
+func (m *Model) handleIntent(intent intents.Intent) tea.Cmd {
+	switch intent := intent.(type) {
+	case intents.AddMessage:
+		id := m.add(intent.Text, intent.Err)
+		if intent.Err == nil && !intent.NoTimeout && id != 0 {
+			expiringMessageTimeout := config.GetExpiringFlashMessageTimeout(config.Current)
+			if expiringMessageTimeout > time.Duration(0) {
+				return tea.Tick(expiringMessageTimeout, func(t time.Time) tea.Msg {
+					return expireMessageMsg{id: id}
+				})
+			}
+		}
+		return nil
+	case intents.DismissOldest:
+		if len(m.messages) == 0 {
+			return nil
+		}
+		m.DeleteOldest()
+		return nil
+	}
+	return nil
+}
+
+func (m *Model) View() []FlashMessageView {
 	messages := m.messages
 	if len(messages) == 0 {
-		return ""
+		return nil
 	}
 
-	var messageBoxes []string
+	y := m.Height - 1
+	var messageBoxes []FlashMessageView
+	// reserve padding and calculate max width for messages
+	maxWidth := m.Width - 4
+
 	for _, message := range messages {
-		style := m.successStyle
+		var text string
+		var style lipgloss.Style
 		if message.error != nil {
+			text = message.error.Error()
 			style = m.errorStyle
-			messageBoxes = append(messageBoxes, style.Render(message.error.Error()))
 		} else {
-			messageBoxes = append(messageBoxes, style.Render(message.text))
+			text = message.text
+			style = m.successStyle
 		}
-	}
-	maxWidth, maxHeight := 0, 0
-	var combined []string
-	for _, box := range messageBoxes {
-		width, height := lipgloss.Size(box)
-		if width > maxWidth {
-			maxWidth = width
-		}
-		if height > maxHeight {
-			maxHeight = height
+
+		// first render without width to check natural size
+		naturalContent := style.Render(text)
+		naturalWidth, _ := lipgloss.Size(naturalContent)
+
+		var content string
+		if naturalWidth <= maxWidth {
+			content = naturalContent
+		} else {
+			// width doesn't fit within maxWidth, set Width for line wrap
+			content = style.Width(maxWidth).Render(text)
 		}
+
+		w, h := lipgloss.Size(content)
+		y -= h
+		messageBoxes = append(messageBoxes, FlashMessageView{
+			Content: content,
+			Rect:    cellbuf.Rect(m.Width-w, y, w, h),
+		})
 	}
-	for _, box := range messageBoxes {
-		combined = append(combined,
-			lipgloss.PlaceHorizontal(maxWidth,
-				lipgloss.Right, box,
-				lipgloss.WithWhitespaceForeground(screen.TransparentFg),
-				lipgloss.WithWhitespaceBackground(screen.TransparentBg),
-			),
-		)
-	}
-	return lipgloss.JoinVertical(lipgloss.Right, combined...)
+	return messageBoxes
 }
 
 func (m *Model) add(text string, error error) uint64 {
@@ -134,6 +173,7 @@ func New(context *context.MainContext) *Model {
 	successStyle := common.DefaultPalette.GetBorder("success", lipgloss.NormalBorder()).Foreground(fg).PaddingLeft(1).PaddingRight(1)
 	errorStyle := common.DefaultPalette.GetBorder("error", lipgloss.NormalBorder()).Foreground(fg).PaddingLeft(1).PaddingRight(1)
 	return &Model{
+		ViewNode:     common.NewViewNode(0, 0),
 		context:      context,
 		messages:     make([]flashMessage, 0),
 		successStyle: successStyle,
