@@ -25,7 +25,7 @@ var (
 	targetToFlags = map[Target]string{
 		TargetAfter:       "--insert-after",
 		TargetBefore:      "--insert-before",
-		TargetDestination: "--destination",
+		TargetDestination: "--onto",
 	}
 )
 
