@@ -110,6 +110,10 @@ func (m *Model) WindowPercentage() float64 {
 	return m.previewWindowPercentage
 }
 
+func (m *Model) YOffset() int {
+	return m.view.YOffset
+}
+
 func (m *Model) Scroll(delta int) tea.Cmd {
 	if delta > 0 {
 		m.view.ScrollDown(delta)
@@ -184,7 +188,12 @@ func (m *Model) Update(msg tea.Msg) tea.Cmd {
 		case tea.MouseButtonWheelRight:
 			m.ScrollHorizontal(scrollAmount)
 		}
-	case common.SelectionChangedMsg, common.RefreshMsg:
+	case common.SelectionChangedMsg:
+		if msg.Item != nil {
+			return m.refreshPreviewForItem(msg.Item)
+		}
+		return m.refreshPreview()
+	case common.RefreshMsg:
 		return m.refreshPreview()
 	case updatePreviewContentMsg:
 		m.SetContent(msg.Content)
@@ -199,6 +208,10 @@ func (m *Model) Update(msg tea.Msg) tea.Cmd {
 			m.view.HalfPageDown()
 		case key.Matches(msg, m.keyMap.Preview.HalfPageUp):
 			m.view.HalfPageUp()
+		case key.Matches(msg, m.keyMap.Preview.Expand):
+			m.Expand()
+		case key.Matches(msg, m.keyMap.Preview.Shrink):
+			m.Shrink()
 		}
 	}
 	return nil
@@ -220,29 +233,39 @@ func (m *Model) reset() {
 }
 
 func (m *Model) refreshPreview() tea.Cmd {
+	return m.refreshPreviewForItem(m.context.SelectedItem)
+}
+
+func (m *Model) refreshPreviewForItem(item common.SelectedItem) tea.Cmd {
 	return common.Debounce(debounceId, debounceDuration, func() tea.Msg {
 		var args []string
 		previewWidth := strconv.Itoa(m.view.Width)
-		switch msg := m.context.SelectedItem.(type) {
-		case context.SelectedFile:
+		switch sel := item.(type) {
+		case common.SelectedFile:
 			args = jj.TemplatedArgs(config.Current.Preview.FileCommand, map[string]string{
 				jj.RevsetPlaceholder:       m.context.CurrentRevset,
-				jj.ChangeIdPlaceholder:     msg.ChangeId,
-				jj.CommitIdPlaceholder:     msg.CommitId,
-				jj.FilePlaceholder:         msg.File,
+				jj.ChangeIdPlaceholder:     sel.ChangeId,
+				jj.CommitIdPlaceholder:     sel.CommitId,
+				jj.FilePlaceholder:         sel.File,
 				jj.PreviewWidthPlaceholder: previewWidth,
 			})
-		case context.SelectedRevision:
+		case common.SelectedRevision:
 			args = jj.TemplatedArgs(config.Current.Preview.RevisionCommand, map[string]string{
 				jj.RevsetPlaceholder:       m.context.CurrentRevset,
-				jj.ChangeIdPlaceholder:     msg.ChangeId,
-				jj.CommitIdPlaceholder:     msg.CommitId,
+				jj.ChangeIdPlaceholder:     sel.ChangeId,
+				jj.CommitIdPlaceholder:     sel.CommitId,
 				jj.PreviewWidthPlaceholder: previewWidth,
 			})
-		case context.SelectedOperation:
+		case common.SelectedCommit:
+			args = jj.TemplatedArgs(config.Current.Preview.EvologCommand, map[string]string{
+				jj.RevsetPlaceholder:       m.context.CurrentRevset,
+				jj.CommitIdPlaceholder:     sel.CommitId,
+				jj.PreviewWidthPlaceholder: previewWidth,
+			})
+		case common.SelectedOperation:
 			args = jj.TemplatedArgs(config.Current.Preview.OplogCommand, map[string]string{
 				jj.RevsetPlaceholder:       m.context.CurrentRevset,
-				jj.OperationIdPlaceholder:  msg.OperationId,
+				jj.OperationIdPlaceholder:  sel.OperationId,
 				jj.PreviewWidthPlaceholder: previewWidth,
 			})
 		}
