@@ -88,7 +88,7 @@ func (m *Model) handleFocusInputMessage(msg tea.Msg) (tea.Cmd, bool) {
 		}
 		if m.oplog != nil {
 			m.oplog = nil
-			return common.SelectionChanged, true
+			return common.SelectionChanged(m.context.SelectedItem), true
 		}
 		return nil, false
 	}
@@ -176,11 +176,6 @@ func (m *Model) Update(msg tea.Msg) tea.Cmd {
 	var cmds []tea.Cmd
 
 	switch msg := msg.(type) {
-	case common.DeferredUpdateMsg:
-		if msg.Fn != nil {
-			return msg.Fn()
-		}
-		return nil
 	case tea.FocusMsg:
 		return tea.Batch(common.RefreshAndKeepSelections, tea.EnableMouseCellMotion)
 	case tea.MouseMsg:
@@ -235,7 +230,7 @@ func (m *Model) Update(msg tea.Msg) tea.Cmd {
 			return tea.Batch(cmds...)
 		case key.Matches(msg, m.keyMap.Quit) && m.isSafeToQuit():
 			return tea.Quit
-		case key.Matches(msg, m.keyMap.OpLog.Mode):
+		case key.Matches(msg, m.keyMap.OpLog.Mode) && m.revisions.InNormalMode():
 			m.oplog = oplog.New(m.context)
 			m.oplog.Parent = m.ViewNode
 			return m.oplog.Init()
@@ -277,13 +272,17 @@ func (m *Model) Update(msg tea.Msg) tea.Cmd {
 				}
 			}
 			m.previewModel.ToggleVisible()
-			cmds = append(cmds, common.SelectionChanged)
+			cmds = append(cmds, common.SelectionChanged(m.context.SelectedItem))
 			return tea.Batch(cmds...)
-		case key.Matches(msg, m.keyMap.Preview.Expand) && m.previewModel.Visible():
-			m.previewModel.Expand()
-			return tea.Batch(cmds...)
-		case key.Matches(msg, m.keyMap.Preview.Shrink) && m.previewModel.Visible():
-			m.previewModel.Shrink()
+		case m.previewModel.Visible() && key.Matches(msg,
+			m.keyMap.Preview.Expand,
+			m.keyMap.Preview.Shrink,
+			m.keyMap.Preview.HalfPageDown,
+			m.keyMap.Preview.HalfPageUp,
+			m.keyMap.Preview.ScrollDown,
+			m.keyMap.Preview.ScrollUp):
+			cmd := m.previewModel.Update(msg)
+			cmds = append(cmds, cmd)
 			return tea.Batch(cmds...)
 		case key.Matches(msg, m.keyMap.CustomCommands):
 			model := customcommands.NewModel(m.context)
@@ -306,6 +305,8 @@ func (m *Model) Update(msg tea.Msg) tea.Cmd {
 		case key.Matches(msg, m.keyMap.QuickSearch) && m.oplog != nil:
 			// HACK: prevents quick search from activating in op log view
 			return nil
+		case key.Matches(msg, m.keyMap.ExecJJ, m.keyMap.ExecShell, m.keyMap.QuickSearch) && m.revisions.InNormalMode():
+			return m.status.Update(msg)
 		case key.Matches(msg, m.keyMap.Suspend):
 			return tea.Suspend
 		default:
@@ -384,7 +385,7 @@ func (m *Model) Update(msg tea.Msg) tea.Cmd {
 		m.stacked = nil
 	case common.ShowPreview:
 		m.previewModel.SetVisible(bool(msg))
-		cmds = append(cmds, common.SelectionChanged)
+		cmds = append(cmds, common.SelectionChanged(m.context.SelectedItem))
 		return tea.Batch(cmds...)
 	case common.TogglePasswordMsg:
 		if m.password != nil {
@@ -406,6 +407,17 @@ func (m *Model) Update(msg tea.Msg) tea.Cmd {
 		m.flash.SetHeight(m.Height)
 	}
 
+	// Unhandled key messages go to the main view (oplog or revisions)
+	// Other messages are broadcast to all models
+	if common.IsInputMessage(msg) {
+		if m.oplog != nil {
+			cmds = append(cmds, m.oplog.Update(msg))
+		} else {
+			cmds = append(cmds, m.revisions.Update(msg))
+		}
+		return tea.Batch(cmds...)
+	}
+
 	cmds = append(cmds, m.revsetModel.Update(msg))
 	cmds = append(cmds, m.status.Update(msg))
 	cmds = append(cmds, m.flash.Update(msg))
